package idv.cjcat.bunnyhill.nodes {
	import flash.display.BitmapData;
	import flash.display3D.Context3DTextureFormat;
	import flash.display3D.Context3DVertexBufferFormat;
	import flash.display3D.IndexBuffer3D;
	import flash.display3D.textures.Texture;
	import flash.display3D.VertexBuffer3D;
	import flash.geom.Point;
	import idv.cjcat.bunnyhill.render.visitor.IRenderVisitor;
	import idv.cjcat.bunnyhill.shaders.basic.BasicShaderProgram;
	import idv.cjcat.bunnyhill.utils.TextureDimensionData;
	import idv.cjcat.bunnyhill.utils.TextureUtils;
	
	public class DrawBitmapData extends Node {
		
		private var _alpha:Number = 1;
		public function get alpha():Number { return _alpha; }
		public function set alpha(value:Number):void {
			_alpha = value;
		}
		
		private var _prevTexWidth:Number;
		private var _prevTexHeight:Number;
		private var _width:Number;
		private var _height:Number;
		private var _maxU:Number;
		private var _maxV:Number;
		private var _source:BitmapData;
		public function draw(source:BitmapData):void {
			//update texture dimension
			var texDimension:TextureDimensionData = TextureUtils.getMinTextureDimension(source.width, source.height);
			_texWidth = texDimension.width;
			_texHeight = texDimension.height;
			_maxU = texDimension.maxU;
			_maxV = texDimension.maxV;
			
			if (_source) {
				
				//dimension mismatch
				if ((_source.width != _texWidth) || (_source.height != _texHeight)) {
					_source.dispose();
					_source = null;
					
					if (_texture) {
						_texture.dispose();
						_texture = null;
					}
				}
			}
			
			if (!_source) {
				_source = new BitmapData(_texWidth, _texHeight, true, 0x00000000);
			}
			
			_source.fillRect(_source.rect, 0x00000000);
			_source.copyPixels(source, source.rect, new Point(), null, null, false);
			_width = source.width;
			_height = source.height;
			
			invalidate();
		}
		
		private var _shader:BasicShaderProgram = new BasicShaderProgram();
		
		private var _texture:Texture;
		private var _textureDirty:Boolean = true;
		private var _texWidth:Number;
		private var _texHeight:Number;
		
		public function DrawBitmapData(source:BitmapData = null) {
			if (source) draw(source);
		}
		
		public function invalidate():void {
			_textureDirty = true;
		}
		
		private var _vertexBuffer:VertexBuffer3D;
		private var _vertexBufferVector:Vector.<Number>;
		private var _indexBuffer:IndexBuffer3D;
		private var _indexBufferVector:Vector.<uint>;
		
		override public function update(visitor:IRenderVisitor):void {
			if (_textureDirty) {
				
				//vertex buffer & index buffer
				//-------------------------------------------------
				
				if (!_vertexBuffer) {
					_vertexBuffer = visitor.graphics.createVertexBuffer(4, 6);
					_indexBuffer = visitor.graphics.createIndexBuffer(6);
					
					_vertexBufferVector = Vector.<Number>([
						0, 0, 0, 1, 0, 0, //top left
						0, 0, 0, 1, 0, 0, //top right
						0, 0, 0, 1, 0, 0, //bottom left
						0, 0, 0, 1, 0, 0  //bottom right
					]);
					
					_indexBufferVector = Vector.<uint>([
						0, 3, 1, 
						0, 2, 3
					]);
				}
				
				//top right
				_vertexBufferVector[6] = _width;
				_vertexBufferVector[10] = _maxU;
				
				//bottom left
				_vertexBufferVector[13] = -_height;
				_vertexBufferVector[17] = _maxV;
				
				//bottom right
				_vertexBufferVector[18] = _width;
				_vertexBufferVector[19] = -_height;
				_vertexBufferVector[22] = _maxU;
				_vertexBufferVector[23] = _maxV;
				
				//upload
				_vertexBuffer.uploadFromVector(_vertexBufferVector, 0, 4);
				_indexBuffer.uploadFromVector(_indexBufferVector, 0, 6);
				
				//-------------------------------------------------
				//end of vertex buffer & index bufffer
				
				
				//texture
				//-------------------------------------------------
				
				if (!_texture) {
					_texture = visitor.graphics.createTexture(_texWidth, _texHeight, Context3DTextureFormat.BGRA, false);
					_prevTexWidth = _texWidth;
					_prevTexHeight = _texHeight;
				}
				
				_texture.uploadFromBitmapData(_source);
				
				//-------------------------------------------------
				//end of texture
				
				
				_textureDirty = false;
			}
		}
		
		override public function render(visitor:IRenderVisitor):void {
			visitor.graphics.shader = _shader;
			_shader.matrix = visitor.matrixStack.top();
			_shader.alpha = _alpha * visitor.alphaStack.top();
			_shader.updateParameters();
			visitor.graphics.shader = _shader;
			
			visitor.graphics.setTextureAt(0, _texture);
			visitor.graphics.setVertexBufferAt(0, _vertexBuffer, 0, Context3DVertexBufferFormat.FLOAT_4);
			visitor.graphics.setVertexBufferAt(1, _vertexBuffer, 4, Context3DVertexBufferFormat.FLOAT_2);
			visitor.graphics.drawTriangles(_indexBuffer);
		}
	}
}