package idv.cjcat.bunnyhill.nodes {
	import flash.utils.Dictionary;
	import idv.cjcat.bunnyhill.render.visitor.IRenderVisitor;
	
	public class Group extends Node implements IGroup {
		
		/** @private */
		protected var _fistEmptyIndex:int = 0;
		/** @private */
		protected var _children:Vector.<INode> = new Vector.<INode>();
		/** @private */
		protected var _childrenIndicies:Dictionary = new Dictionary();
		
		public function numChildren():int { return _fistEmptyIndex };
		
		public function Group(...children) {
			for (var i:int = 0, len:int = children.length; i < len; ++i) {
				add(children[i]);
			}
		}
		
		
		//children management
		//------------------------------------------------------------------------------------------------
		
		public function add(child:INode):void {
			if (_childrenIndicies[child] !== undefined) return;
			
			//expand vector if full
			_children[_fistEmptyIndex] = child;
			_childrenIndicies[child] = _fistEmptyIndex;
			if (++_fistEmptyIndex == _children.length) _children.length <<= 1;
		}
		
		public function remove(child:INode):void {
			if (_childrenIndicies[child] !== undefined) {
				
				//remove from vector
				var index:int = _childrenIndicies[child];
				_children[index] = _children[--_fistEmptyIndex];
				_children[_fistEmptyIndex] = null;
				delete _childrenIndicies[child];
			}
		}
		
		public function clear():void {
			var child:INode;
			for (var i:int = 0; i < _fistEmptyIndex; ++i) {
				child = _children[i];
				
				//clear record
				delete _childrenIndicies[child];
				
				//empty vector element
				_children[i] = null;
			}
			
			_fistEmptyIndex = 0;
		}
		
		//------------------------------------------------------------------------------------------------
		//end of children management
		
		
		//children visiting
		//------------------------------------------------------------------------------------------------
		
		private var child:INode;
		override public function render(visitor:IRenderVisitor):void {
			for (var i:int = 0; i < _fistEmptyIndex; ++i) {
				child = _children[i];
				
				//update index dictionary
				_childrenIndicies[child] = i;
				
				//visit child
				child.visit(visitor);
			}
		}
	}
}