package idv.cjcat.bunnyhill.nodes {
	import flash.geom.Matrix3D;
	import idv.cjcat.bunnyhill.geom.Vec2D;
	import idv.cjcat.bunnyhill.render.visitor.IRenderVisitor;
	import idv.cjcat.bunnyhill.utils.BunnyhillMath;
	
	public class Transform extends Node {
		
		//matrix
		//------------------------------------------------------------------------------------------------
		
		private var _matrix:Matrix3D = new Matrix3D();
		
		private var _matrixDirty:Boolean;
		private var _center:Vec2D;
		private var _position:Vec2D;
		private var _rotation:Number;
		private var _scale:Vec2D;
		
		public function get center():Vec2D { return _center; }
		
		public function get position():Vec2D { return _position; }
		
		public function get rotation():Number { return _rotation; }
		public function set rotation(value:Number):void {
			_rotation = value;
			invalidateMatrix();
		}
		
		public function get scale():Vec2D { return _scale; }
		
		private function invalidateMatrix(vec:Vec2D = null):void {
			_matrixDirty = true;
		}
		
		//------------------------------------------------------------------------------------------------
		//end of matrix
		
		
		public var child:INode;
		
		public function Transform(child:INode = null) {
			this.child = child;
			
			_center = new Vec2D(0, 0);
			_position = new Vec2D(0, 0);
			_rotation = 0;
			_scale = new Vec2D(1, 1);
			
			_center.onChange.add(invalidateMatrix);
			_position.onChange.add(invalidateMatrix);
			_scale.onChange.add(invalidateMatrix);
		}
		
		private var _rawData:Vector.<Number> = new Vector.<Number>(16, true);
		override public function update(visitor:IRenderVisitor):void {
			if (_matrixDirty) {
				var theta:Number = BunnyhillMath.DEGREE_TO_RADIAN * _rotation;
				var cos:Number = Math.cos(theta);
				var sin:Number = Math.sin(theta);
				var sx:Number = scale.x;
				var sy:Number = scale.y;
				var cx:Number = _center.x;
				var cy:Number = _center.y;
				var px:Number = _position.x;
				var py:Number = _position.y;
				
				//rotation
				_rawData[0] = cos * sx;
				_rawData[1] = -sin * sx;
				_rawData[4] = sin * sy;
				_rawData[5] = cos * sy;
				
				//translation
				_rawData[12] = -cos * sx * cx + sin * sy * cy + px;
				_rawData[13] = sin * sx * cx + cos * sy * cy - py;
				
				//zeros
				_rawData[2] = 
				_rawData[3] = 
				_rawData[6] = 
				_rawData[7] = 
				_rawData[8] = 
				_rawData[9] = 
				_rawData[11] = 
				_rawData[14] = 0;
				
				//ones
				_rawData[10] = _rawData[15] = 1;
				
				_matrix.rawData = _rawData;
				
				_matrixDirty = false;
			}
		}
		
		override public function render(visitor:IRenderVisitor):void {
			visitor.matrixStack.pushPrepend(_matrix);
			if (child) child.visit(visitor);
			visitor.matrixStack.pop();
		}
	}
}