package idv.cjcat.bunnyhill.render {
	import flash.display.Stage;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.display3D.Context3D;
	import flash.display3D.Context3DBlendFactor;
	import flash.display3D.Context3DCompareMode;
	import flash.display3D.Context3DRenderMode;
	import flash.events.Event;
	import flash.geom.Rectangle;
	import idv.cjcat.bunnyhill.IDisposable;
	import idv.cjcat.bunnyhill.nodes.INode;
	import idv.cjcat.bunnyhill.render.blending.BlendMode;
	import idv.cjcat.bunnyhill.render.targets.BackBufferRenderTarget;
	import idv.cjcat.bunnyhill.render.targets.IRenderTarget;
	import idv.cjcat.bunnyhill.render.visitor.RenderVisitor;
	import idv.cjcat.signals.ISignal;
	import idv.cjcat.signals.Signal;
	
	public final class Renderer implements IDisposable {
		
		//signals
		//------------------------------------------------------------------------------------------------
		
		private var _onReady:ISignal = new Signal(Renderer);
		public function get onReady():ISignal { return _onReady; }
		
		//------------------------------------------------------------------------------------------------
		//end of signals
		
		
		public var rootNode:INode;
		
		private var _width:Number;
		public function get width():Number { return _width; }
		private var _height:Number;
		public function get height():Number { return _height; }
		
		private var _antiAlias:int = 0;
		public function get antiAlias():int { return _antiAlias; }
		public function set antiAlias(value:int):void {
			if (_antiAlias != value) {
				_antiAlias = value;
			}
			resetContext3D();
		}
		
		private var _stage:Stage;
		private var _context3D:Context3D;
		private var _visitor:RenderVisitor;
		
		/*
		public static function setupOnStage(stage:Stage, antiAlias:int = 0, autoResize:Boolean = true):Renderer {
			var renderer:Renderer = new Renderer();
			renderer.setupOnStage(stage, antiAlias, autoResize);
			return renderer;
		}
		*/
		
		public function Renderer() {
			
		}
		
		public function setupOnStage(stage:Stage, antiAlias:int = 0, autoResize:Boolean = true):void {
			_stage = stage;
			_stage.scaleMode = StageScaleMode.NO_SCALE;
			_stage.align = StageAlign.TOP_LEFT;
			
			_antiAlias = antiAlias;
			
			_width = _stage.stageWidth;
			_height = _stage.stageHeight;
			
			if (autoResize) _stage.addEventListener(Event.RESIZE, onStageResize);
			
			_stage.stage3Ds[0].addEventListener(Event.CONTEXT3D_CREATE, onContext3DCreate);
			_stage.stage3Ds[0].viewPort = new Rectangle(0, 0, _width, _height);
			_stage.stage3Ds[0].requestContext3D(Context3DRenderMode.AUTO);
		}
		
		private function onContext3DCreate(e:Event):void {
			_onReady.dispatch(this);
			resetContext3D();
		}
		
		
		private var _defaultRenderTarget:IRenderTarget = new BackBufferRenderTarget();
		
		public function render(time:Number = -1):void {
			_visitor.timestamp++;
			if (rootNode) {
				if (time <= 0) time = 1 / _stage.frameRate;
				
				//visitor initialization
				_visitor.timeStep = time;
				_visitor.reset(_width, _height, _antiAlias);
				_visitor.graphics.blendMode = BlendMode.ALPHA;
				_visitor.graphics.renderTarget = _defaultRenderTarget;
				
				rootNode.visit(_visitor);
				
				_visitor.graphics.present();
			}
		}
		
		public function dispose():void {
			
		}
		
		private function onStageResize(e:Event):void {
			var stage:Stage = Stage(e.target);
			
			_width = stage.stageWidth;
			_height = stage.stageHeight;
			_visitor.onStageResize.dispatch(_width, _height);
			
			stage.stage3Ds[0].viewPort = new Rectangle(0, 0, _width, _height);
			
			resetContext3D();
		}
		
		private function resetContext3D():void {
			if (_visitor) _visitor.dispose();
			
			_context3D = _stage.stage3Ds[0].context3D;
			_visitor = new RenderVisitor(_context3D);
			_context3D.configureBackBuffer(_width, _height, _antiAlias, true);
			_context3D.setDepthTest(true, Context3DCompareMode.LESS_EQUAL);
		}
	}
}