package idv.cjcat.bunnyhill.render.buffer {
	import flash.display3D.Context3D;
	import flash.utils.Dictionary;
	import idv.cjcat.bunnyhill.IDisposable;
	import idv.cjcat.bunnyhill.utils.TextureDimensionData;
	import idv.cjcat.bunnyhill.utils.TextureUtils;
	
	public class RenderBufferPool implements IRenderBufferPool {
		
		private var _poolUnits:Dictionary;
		private var _context3D:Context3D;
		
		public function RenderBufferPool(context3D:Context3D) {
			_poolUnits = new Dictionary();
			_context3D = context3D;
		}
		
		public function get(width:int, height:int):IRenderBuffer {
			var dimension:TextureDimensionData = 
				TextureUtils.getMinTextureDimension(width, height);
				
			var poolUnit:RenderBufferPoolUnit;
			if ((poolUnit = _poolUnits[getKey(dimension.width, dimension.height)]) == null) {
				_poolUnits[getKey(dimension.width, dimension.height)] = 
				poolUnit = 
				new RenderBufferPoolUnit(
					_context3D, 
					dimension.width, 
					dimension.height
				);
			}
			
			return poolUnit.get();
		}
		
		public function recycle(renderBuffer:IRenderBuffer):void {
			var poolUnit:RenderBufferPoolUnit = _poolUnits[getKey(renderBuffer.width, renderBuffer.height)];
			poolUnit.recycle(renderBuffer);
		}
		
		public function dispose():void {
			for (var key:* in _poolUnits) {
				var poolUnit:RenderBufferPoolUnit = _poolUnits[key];
				poolUnit.dispose();
				delete _poolUnits[key];
			}
			
			_poolUnits = null;
		}
		
		private function getKey(width:int, height:int):String {
			return width.toString() + "X" + height.toString();
		}
	}
}