package idv.cjcat.bunnyhill.render.visitor {
	import flash.display.BitmapData;
	import flash.display3D.Context3D;
	import flash.display3D.IndexBuffer3D;
	import flash.display3D.textures.Texture;
	import flash.display3D.textures.TextureBase;
	import flash.display3D.VertexBuffer3D;
	import flash.geom.Rectangle;
	import idv.cjcat.bunnyhill.render.blending.IBlendMode;
	import idv.cjcat.bunnyhill.render.targets.IRenderTarget;
	import idv.cjcat.bunnyhill.shaders.IShaderProgram;
	
	public class Graphics implements IGraphics {
		
		private var _context3D:Context3D;
		public function get context3D():Context3D { return _context3D; }
		
		private var _visitor:IRenderVisitor;
		
		public function Graphics(context3D:Context3D, visitor:IRenderVisitor) {
			_context3D = context3D;
			_visitor = visitor;
		}
		
		
		//vertices
		//------------------------------------------------------------------------------------------------
		
		public function createVertexBuffer(numVertices:int, data32PerVertex:int):VertexBuffer3D {
			return _context3D.createVertexBuffer(numVertices, data32PerVertex);
		}
		
		public function createIndexBuffer(numIndices:int):IndexBuffer3D {
			return _context3D.createIndexBuffer(numIndices);
		}
		
		//------------------------------------------------------------------------------------------------
		//end of vertices
		
		
		//textures
		//------------------------------------------------------------------------------------------------
		
		public function beginRenderToTexture(texture:TextureBase, enableDepthAndStencil:Boolean = false, antiAlias:int = 0, surfaceSelector:int = 0):void {
			_context3D.setRenderToTexture(texture, enableDepthAndStencil, antiAlias, surfaceSelector);
		}
		
		public function endRenderToTexture():void {
			_context3D.setRenderToBackBuffer();
		}
		
		public function createTexture(width:int, height:int, format:String, optimizeForRenderToTexture:Boolean = false):Texture {
			return _context3D.createTexture(width, height, format, optimizeForRenderToTexture);
		}
		
		public function setTextureAt(index:int, texture:TextureBase):void {
			_context3D.setTextureAt(index, texture);
		}
		
		//------------------------------------------------------------------------------------------------
		//end of textures
		
		
		//drawing
		//------------------------------------------------------------------------------------------------
		
		public function setVertexBufferAt(index:int, buffer:VertexBuffer3D, bufferOffset:int = 0, format:String = "float4"):void {
			_context3D.setVertexBufferAt(index, buffer, bufferOffset, format);
		}
		
		public function drawTriangles(indexBuffer:IndexBuffer3D, firstIndex:int = 0, numTriangles:int = -1):void {
			_context3D.drawTriangles(indexBuffer, firstIndex, numTriangles);
		}
		
		//------------------------------------------------------------------------------------------------
		//end of drawing
		
		
		//rendering
		//------------------------------------------------------------------------------------------------
		
		public function clear(red:Number = 0, green:Number = 0, blue:Number = 0, alpha:Number = 1, depth:Number = 1, stencil:uint = 0, mask:uint = 0xFFFFFFFF):void {
			_context3D.clear(red, green, blue, alpha, depth, stencil, mask);
		}
		
		public function present():void {
			_context3D.present();
		}
		
		public function drawToBitmapData(bitmapData:BitmapData):void {
			_context3D.drawToBitmapData(bitmapData);
		}
		
		public function setBlendFactors(sourceFactor:String, destinationFactor:String):void {
			_context3D.setBlendFactors(sourceFactor, destinationFactor);
		}
		
		private var _blendMode:IBlendMode;
		public function get blendMode():IBlendMode { return _blendMode; }
		public function set blendMode(value:IBlendMode):void {
			_blendMode = value;
			_blendMode.setup(_context3D);
		}
		
		private var _renderTarget:IRenderTarget;
		public function get renderTarget():IRenderTarget { return _renderTarget; }
		public function set renderTarget(value:IRenderTarget):void {
			_renderTarget = value;
			_renderTarget.setup(_context3D, true, _visitor.antiAlias);
		}
		
		public function setScissorRectangle(rect:Rectangle):void {
			_context3D.setScissorRectangle(rect);
		}
		
		//------------------------------------------------------------------------------------------------
		//end of rendering
		
		
		//shaders
		//------------------------------------------------------------------------------------------------
		
		private var _shader:IShaderProgram;
		public function get shader():IShaderProgram { return _shader; }
		public function set shader(value:IShaderProgram):void {
			_shader = value;
			_shader.setup(_context3D);
		}
		
		//------------------------------------------------------------------------------------------------
		//end of shaders
		
		
		//dispose
		//------------------------------------------------------------------------------------------------
		
		public function dispose():void {
			_context3D = null;
			_visitor = null;
		}
		
		//------------------------------------------------------------------------------------------------
		//end of dispose
	}
}