package idv.cjcat.bunnyhill.render.visitor {
	import flash.display.BitmapData;
	import flash.display3D.Context3D;
	import flash.display3D.IndexBuffer3D;
	import flash.display3D.textures.Texture;
	import flash.display3D.textures.TextureBase;
	import flash.display3D.VertexBuffer3D;
	import flash.geom.Rectangle;
	import idv.cjcat.bunnyhill.IDisposable;
	import idv.cjcat.bunnyhill.render.blending.IBlendMode;
	import idv.cjcat.bunnyhill.render.targets.IRenderTarget;
	import idv.cjcat.bunnyhill.shaders.IShaderProgram;
	
	public interface IGraphics extends IDisposable {
		
		//context
		//------------------------------------------------------------------------------------------------
		
		function get context3D():Context3D;
		
		//------------------------------------------------------------------------------------------------
		//end of context
		
		
		//vertices
		//------------------------------------------------------------------------------------------------
		
		function createVertexBuffer(numVertices:int, data32PerVertex:int):VertexBuffer3D;
		function createIndexBuffer(numIndices:int):IndexBuffer3D;
		
		//------------------------------------------------------------------------------------------------
		//end of vertices
		
		
		//textures
		//------------------------------------------------------------------------------------------------
		
		function beginRenderToTexture(texture:TextureBase, enableDepthAndStencil:Boolean = false, antiAlias:int = 0, surfaceSelector:int = 0):void;
		function endRenderToTexture():void;
		function createTexture(width:int, height:int, format:String, optimizeForRenderToTexture:Boolean = false):Texture;
		function setTextureAt(index:int, texture:TextureBase):void;
		
		//------------------------------------------------------------------------------------------------
		//end of textures
		
		
		//drawing
		//------------------------------------------------------------------------------------------------
		
		function setVertexBufferAt(index:int, buffer:VertexBuffer3D, bufferOffset:int = 0, format:String = "float4"):void;
		function drawTriangles(indexBuffer:IndexBuffer3D, firstIndex:int = 0, numTriangles:int = -1):void;
		
		//------------------------------------------------------------------------------------------------
		//end of drawing
		
		
		//rendering
		//------------------------------------------------------------------------------------------------
		
		function clear(red:Number = 0, green:Number = 0, blue:Number = 0, alpha:Number = 1, depth:Number = 1, stencil:uint = 0, mask:uint = 0xFFFFFFFF):void;
		function present():void;
		function drawToBitmapData(bitmapData:BitmapData):void;
		function setBlendFactors(sourceFactor:String, destinationFactor:String):void;
		function get blendMode():IBlendMode;
		function set blendMode(value:IBlendMode):void;
		function get renderTarget():IRenderTarget;
		function set renderTarget(value:IRenderTarget):void;
		function setScissorRectangle(rect:Rectangle):void;
		
		//------------------------------------------------------------------------------------------------
		//end of rendering
		
		
		//shaders
		//------------------------------------------------------------------------------------------------
		
		function get shader():IShaderProgram;
		function set shader(value:IShaderProgram):void;
		
		//------------------------------------------------------------------------------------------------
		//end of shaders
	}
}