package idv.cjcat.bunnyhill.render.visitor {
	import flash.geom.Matrix3D;
	import idv.cjcat.bunnyhill.errors.MatrixStackError;
	
	public final class MatrixStack implements IMatrixStack {
		
		private var _stack:Vector.<Matrix3D>;
		private var _topMatrixIndex:int;
		private var _firstEmptyIndex:int;
		
		private var _temp:Matrix3D = new Matrix3D();
		
		public function MatrixStack() {
			reset();
		}
		
		public final function reset(bottomMatrix:Matrix3D = null):void {
			if (!bottomMatrix) bottomMatrix = new Matrix3D();
			
			_stack = Vector.<Matrix3D>([bottomMatrix.clone()]);
			_topMatrixIndex = 0;
			_firstEmptyIndex = 1;
		}
		
		public final function top():Matrix3D {
			return _stack[_topMatrixIndex];
		}
		
		public final function push(matrix:Matrix3D):void {
			//expand matrix vector when full
			if (++_topMatrixIndex == _stack.length) _stack.length <<= 1;
			if (_topMatrixIndex == _firstEmptyIndex) {
				_stack[_topMatrixIndex] = new Matrix3D();
				++_firstEmptyIndex;
			}
			matrix.copyToMatrix3D(_stack[_topMatrixIndex]);
		}
		
		public final function pushAppend(matrix:Matrix3D):void {
			top().copyToMatrix3D(_temp);
			_temp.append(matrix);
			push(_temp);
		}
		
		public final function pushPrepend(matrix:Matrix3D):void {
			top().copyToMatrix3D(_temp);
			_temp.prepend(matrix);
			push(_temp);
		}
		
		public final function pop():void {
			if (_topMatrixIndex > 0) --_topMatrixIndex;
			else throw new MatrixStackError(this, "No more matrix can be popped.");
		}
		
		public function dispose():void {
			for (var i:int = 0, len:int = _stack.length; i < len; ++i) {
				_stack[i] = null;
			}
			_stack = null;
		}
	}
}