package idv.cjcat.bunnyhill.render.visitor {
	import flash.display3D.Context3D;
	import flash.geom.Matrix3D;
	import idv.cjcat.bunnyhill.render.buffer.IRenderBufferPool;
	import idv.cjcat.bunnyhill.render.buffer.RenderBufferPool;
	import idv.cjcat.signals.ISignal;
	import idv.cjcat.signals.Signal;
	
	public class RenderVisitor implements IRenderVisitor {
		
		//signals
		//------------------------------------------------------------------------------------------------
		
		private var _onStageResize:ISignal = new Signal(Number, Number);
		public function get onStageResize():ISignal { return _onStageResize; }
		
		//------------------------------------------------------------------------------------------------
		//end of signals
		
		private var _stageWidth:Number;
		public function get stageWidth():Number { return _stageWidth; }
		private var _stageHeight:Number;
		public function get stageHeight():Number { return _stageHeight; }
		private var _antiAlias:int;
		public function get antiAlias():int { return _antiAlias; }
		
		public function reset(stageWidth:Number, stageHeight:Number, antiAlias:int):void {
			//stage dimension
			_stageWidth = stageWidth;
			_stageHeight = stageHeight;
			
			//matrix stack
			var bottomMatrix:Matrix3D = new Matrix3D();
			bottomMatrix.appendScale(2 / _stageWidth, 2 / stageHeight, 1);
			bottomMatrix.appendTranslation(-1, 1, 0);
			_matrixStack.reset(bottomMatrix);
			
			//alpha stack
			_alphaStack.reset(1);
		}
		
		private var _frame:int;
		public function get frame():int { return _frame; }
		public function nextFrame():int { return ++_frame; }
		public function resetFrame():int { return _frame = 0 }
		
		/** @private */
		internal var _timeStep:Number;
		public function get timeStep():Number { return _timeStep; }
		public function set timeStep(value:Number):void {
			_timeStep = value;
		}
		
		/** @private */
		internal var _timestamp:int;
		public function get timestamp():int { return _timestamp; }
		public function set timestamp(value:int):void { _timestamp = value; }
		
		private var _graphics:IGraphics;
		public function get graphics():IGraphics { return _graphics; }
		
		private var _alphaStack:INumberStack;
		public function get alphaStack():INumberStack { return _alphaStack; }
		
		private var _matrixStack:IMatrixStack;
		public function get matrixStack():IMatrixStack { return _matrixStack; }
		
		private var _renderBufferPool:IRenderBufferPool;
		public function get renderBufferPool():IRenderBufferPool { return _renderBufferPool; }
		
		public function RenderVisitor(context3D:Context3D) {
			_frame = 0;
			_timestamp = 0;
			_graphics = new Graphics(context3D, this);
			_alphaStack = new NumberStack();
			_matrixStack = new MatrixStack();
			_renderBufferPool = new RenderBufferPool(context3D);
		}
		
		public function dispose():void {
			_graphics.dispose();
			_alphaStack.dispose();
			_matrixStack.dispose();
			_renderBufferPool.dispose();
			
			_graphics = null;
			_alphaStack = null;
			_matrixStack = null;
			_renderBufferPool = null;
		}
	}
}